////////////////////////////////////////////////////////////////////////////////
//Copyright (c) 2006
//	Adam Sarzyski
//	adamusx@tlen.pl
//  adam@wobit.com.pl
////////////////////////////////////////////////////////////////////////////////


// ====== SONAR V2 ==============

#include "Main.h"

// Przykadowe funkcjie obsugi komend I2C sterujacych sonarami 

// Miedzy wykonaniem pomiaru przez Sonar a odczytaniem wyniku musi byc 
// Conajmniej 30ms przerwy. Wynika to z tego ze timeout sonara jest ustawiony na 
// dystans maks 5m (10m z powrotem echa). Dystans ten dzwiek pokonuje w  okoo 30ms.
// Dla trybu auto czas ten powinien by jeszcze wiekszy  i wynosic conajmniej 200ms
// Odebrana wartosc jest w mm. 
// Jesli sonar nie odbierze echa wysya wartosc 30000. 
 


// Funkcja rozkazujaca sonarowi wykonanie pomiary w trybie manualnym
// Argumenty wejsciowe :Wzmocnienie(1-32), Adres sonara
void sonar_request(unsigned char SONAR_ADR)
{
	 
		//unsigned char SLAVE_ADR = SONAR_ADR;
		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
		messageBuf[1] = TWI_CMD_MASTER_WRITE;
		messageBuf[2] = TWI_CMD_manual_mes;
		//messageBuf[3] = 0;           // dla zgodnosci z poprzednia wersja sonaru 
                                     // wysylamy 3 bajty             
		TWI_Start_Transceiver_With_Data(messageBuf,3);
	 	
}

// Funkcja ustawiajaca ilosc impulsow generowanych w jednej paczce przez sonar
// Argumenty wejsciowe :Ilosc impulsow (1-32), Adres sonara
void sonar_set_pulse_number(unsigned char pulse_number,unsigned char SONAR_ADR)
{
	 
		//unsigned char SLAVE_ADR = sonar_addr;
		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
		messageBuf[1] = TWI_CMD_MASTER_WRITE;
		messageBuf[2] = TWI_CMD_pulse_number;
		messageBuf[3] = pulse_number;
		TWI_Start_Transceiver_With_Data(messageBuf,4);
		
}

// Funkcja ustawiajaca maksymalny zasieg sonara
// Argumenty wejsciowe :Zasieg w mm ( 100 - 5000), Adres sonara
void sonar_set_max_distance(unsigned int distance,unsigned char SONAR_ADR)
{	 
		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
		messageBuf[1] = TWI_CMD_MASTER_WRITE;
		messageBuf[2] = TWI_CMD_max_distance;
		messageBuf[3] = (unsigned char)distance;
		messageBuf[4] = (unsigned char)(distance>>8);
		TWI_Start_Transceiver_With_Data(messageBuf,5);		
}

// Funkcja ustawiajaca poziom czulosci sonara
// Domyslna wartosc 0. W przypadku problemow ze zbyt duza czuloscia 
// mozna zmienic wartosc w zakresie 0 - 40
// 0 - najwieksza czulosc
// Argumenty wejsciowe :czulosc (0 - 40), Adres sonara
void sonar_set_sense(unsigned char sense,unsigned char SONAR_ADR)
{	 
		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
		messageBuf[1] = TWI_CMD_MASTER_WRITE;
		messageBuf[2] = TWI_CMD_compare_level;
		messageBuf[3] = sense;
		TWI_Start_Transceiver_With_Data(messageBuf,4);		
}

// Funkcja ustawiajaca poziom progu wyjscia OUT
// Domyslna wartosc 200mm .
// Argumenty wejsciowe : Prog  (60 - 5000), Adres sonara
void sonar_set_treshold_level(unsigned int treshold_level,unsigned char SONAR_ADR)
{	 
		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
		messageBuf[1] = TWI_CMD_MASTER_WRITE;
		messageBuf[2] = TWI_CMD_threshold_level;
		messageBuf[3] = (unsigned char)treshold_level;
		messageBuf[4] = (unsigned char)(treshold_level>>8);
		TWI_Start_Transceiver_With_Data(messageBuf,5);		
}

// Uspienie sonara
// Argumenty wejsciowe : Adres sonara
void sonar_sleep(unsigned char SONAR_ADR)
{	 
		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
		messageBuf[1] = TWI_CMD_MASTER_WRITE;
		messageBuf[2] = TWI_CMD_sleep;
		TWI_Start_Transceiver_With_Data(messageBuf,3);		
}

// Automatyczne uspienie sonara
// Argumenty wejsciowe : Auto ON:1, OFF:0,  Adres sonara
void sonar_auto_sleep(unsigned char AutoSleep_ON_OFF, unsigned char SONAR_ADR)
{	 
		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
		messageBuf[1] = TWI_CMD_MASTER_WRITE;
		messageBuf[2] = TWI_CMD_auto_sleep;
		messageBuf[3] = AutoSleep_ON_OFF;
		TWI_Start_Transceiver_With_Data(messageBuf,4);		
}

// Funkcja odbierajaca zmierzona odleglosc przez sonar 
// Argumenty wejsciowe: Adres sonara
// Zwracana wartosc: Odleglosc w mm. 
// Jesli TWI zajete,zwraca wartosc 0 
int sonar_gets_distance(unsigned char SONAR_ADR)
{
 	//unsigned char SLAVE_ADR = sonar_addr; 
	if ( ! TWI_Transceiver_Busy() )
	 {
			messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); //TWI slave adres + read bit			
			messageBuf[1] = TWI_CMD_MASTER_READ;
			             						
		        
				TWI_Start_Transceiver_With_Data( messageBuf, 2 );
				
			    while(TWI_Transceiver_Busy()); 
			    _delay_us(100);
	
				if (TWI_statusReg.lastTransOK )
				{
			 		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT); 
					TWI_Start_Transceiver_With_Data( messageBuf, 3 );        
					TWI_Get_Data_From_Transceiver( messageBuf, 3 );		// odebranie danych z sonara
					wynik=messageBuf[1] | (messageBuf[2]<<8);			// odczyt odebranych danch z bufora
			    	return (wynik);
				}
		      
	}
	
	 		return 0;
}

// Funkcja oodczytujaca aktualne parammetry sonara 
// Argumenty wejsciowe: Adres sonara

void sonar_gets_parameters(unsigned char SONAR_ADR)
{
 	//unsigned char SLAVE_ADR = sonar_addr; 
	if ( ! TWI_Transceiver_Busy() )
	 {
			messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); //TWI slave adres + read bit
			
			messageBuf[1] = TWI_CMD_CHECK_PARAM;
			             							// komenda TWI
		     {   
				TWI_Start_Transceiver_With_Data( messageBuf, 2 );
				
			    while(TWI_Transceiver_Busy()); 
			    _delay_us(100);
			    
			 	
				if (TWI_statusReg.lastTransOK )
				{
			 		messageBuf[0] = (SONAR_ADR<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT); 
					TWI_Start_Transceiver_With_Data( messageBuf, 3 );        		  // odebranie danych z sonara
					TWI_Get_Data_From_Transceiver( messageBuf, 3 );                   // odczyt odebranych danch z bufora
					
					// Tutaj odbieramy aktualne wartosci wzmocnienia i ilosci impulsow 
					//np :
					 wzmocnienie=  messageBuf[1]; 	// odczytane wzmocnienie sonara 
					 ilosc_impulsow= messageBuf[2]; // odczytane ilosc impulsow sonara 		
			    	
				}
              }
	}
	
	 	
}
