////////////////////////////////////////////////////////////////////////////////
//Copyright (c) 2005
//	Lukasz Matecki
//	lutecki@op.pl
////////////////////////////////////////////////////////////////////////////////
//December 28, 2005 10:08:45 AM 
////////////////////////////////////////////////////////////////////////////////
#include "Main.h"

#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
   #error RX buffer size is not a power of 2
#endif

#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )
#if ( UART_TX_BUFFER_SIZE & UART_TX_BUFFER_MASK )
   #error TX buffer size is not a power of 2
#endif
////////////////////////////////////////////////////////////////////////////////
/* Static Variables */
static unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
static unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
////////////////////////////////////////////////////////////////////////////////

void UART_init(void)//inicjalizacja komunikacji
{
unsigned char x;
   
	//UBRRH=0x01;
	//UBRRL=0xA0;		//dla 2400@16MHz
	//UBRRL=207;		//dla 4800@16MHz
	//UBRRL=103;		//dla 9600@16MHz
	//UBRRL=51;			//dla 19200@16MHz
	//UBRRL=16;			//dla 57600@16MHz
	UBRRL=8;			//dla 57600@8MHz
	
	//UBRRL=8;			//dla 115200@16MHz
   
   UCSRB=0x98;//10011000
   UCSRC=0x86;
   
   x = 0;			   /* Flush receive buffer */
   UART_RxTail = x;
   UART_RxHead = x;
   UART_TxTail = x;
   UART_TxHead = x;
   
   uart_echo=0;
   
	#ifdef XON_XOFF
		UART_clear_to_send=1;
		UART_transmit_control(UART_XON);
		UART_request_to_send=1;
	#endif
}
////////////////////////////////////////////////////////////////////////////////
//Przerwanie od odebranego znaku
////////////////////////////////////////////////////////////////////////////////
SIGNAL(SIG_UART_RECV)
{
   
	static unsigned char data;
	static unsigned char tmphead;

	data = UDR; 						//przepisz rejestr do data
	
	#ifdef XON_XOFF
	
		if(data==UART_XOFF) 	
		{
			UCSRB&=~(1<<UDRIE);
			return;
		}
		if(data==UART_XON)
		{
			UCSRB|=(1<<UDRIE);
			return;
		}
	
		if(UART_rx_buffer_full())
		{
			UART_transmit_control(UART_XOFF);
		}
	#endif
   	
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
									//oblicz wskasnik bufora
	UART_RxHead = tmphead;		

	if ( tmphead == UART_RxTail )	//obsluga bledu przepelnienia bufora
	{
		UART_flush_rx_buffer();  
	}

	else
	{
	UART_RxBuf[tmphead] = data; 	//zapisz dane w buforze odbiorczym
	}
}
////////////////////////////////////////////////////////////////////////////////
//Przerwanie od pustego rejestru nadawczego (wywolywane jest cyklicznie
//jesli dane zostaly wyslane, dlatego jest niezbedne jego wlaczanie/wylaczanie)
////////////////////////////////////////////////////////////////////////////////
SIGNAL(SIG_UART_DATA)
{
	
	static unsigned char tmptail;
	static unsigned char temp;
	
	if ( UART_TxHead != UART_TxTail )//sprawdz czy wyslano juz wszystkie dane
	{
		tmptail = ( UART_TxTail + 1 ) & UART_TX_BUFFER_MASK;
		UART_TxTail = tmptail;	  
		temp = UART_TxBuf[tmptail];	//rozpocznij transmisje
		UDR = temp;
		
		#ifdef PILOT				//wykorzystywane w puplicie sterowniczym
			sei();
			LCD_putc(temp);
		#endif
	}
	else
	{
	  	UCSRB&=~(1<<UDRIE);			//wylacz przerwania UDRE
	}
}
////////////////////////////////////////////////////////////////////////////////
//Odbieranie danych - pobieranie z bufora
////////////////////////////////////////////////////////////////////////////////
inline unsigned char UART_receive_byte( void )
{
	unsigned char tmptail;
	
	#ifdef XON_XOFF					//jesli sterowanie przeplywem
		if(UART_rx_buffer_empty())	//sygnalizuj gotowosc 
	 	{	
	 		UART_transmit_control(UART_XON);
	 	}
	 	while(UART_RxHead == UART_RxTail);
	 								//czekaj na dane w buforze
	#else
		while(UART_RxHead == UART_RxTail);
	#endif
	
   	tmptail = ( UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;
   									//pobierz dane z bufora
   	UART_RxTail = tmptail;		
   
	return UART_RxBuf[tmptail]; 
}
////////////////////////////////////////////////////////////////////////////////
//Wysylanie bajtu poprzez wpisanie do bufora nadawczeo i uruchomienie przerwania
//od pustego rejestru danej do wyslania
////////////////////////////////////////////////////////////////////////////////
inline void UART_transmit_byte( unsigned char data )
{
	unsigned char tmphead;
	
   	tmphead = ( UART_TxHead + 1 ) & UART_TX_BUFFER_MASK; 
   									
   	while ( tmphead == UART_TxTail );
   									//czekaj na wolne miejsce w buforze
   	
   	UART_TxBuf[tmphead] = data;		//zapisz dane w buforze
   	UART_TxHead = tmphead;			

   	UCSRB|=(1<<UDRIE);				//uruchom przerwania
}
////////////////////////////////////////////////////////////////////////////////
//Wysylanie bajtu kontrolnego - poza kolejnoscia w buforze.
//Funkcja ma na celu przeslanie ASAP pojedynczego bajtu, wylaczenie przerwan
//powoduje, ze natychmiast po zwolnieniu interfejsu jest do niego zaladowany bajt.
//Operacja nie jest buforowana, takze nie ma wplywu na bufor nadawczy.
////////////////////////////////////////////////////////////////////////////////
inline void UART_transmit_control(unsigned char data)
{
	static unsigned char sreg;
	static unsigned char UDRIE_state;
	
	sreg=SREG;						//zachowanie stanu rejestru statusowego
	cli();							//wylaczenie przerwan
	
	UDRIE_state=UCSRB&(1<<UDRIE);	//zachowanie stanu bitu zezw. na przerwanie
	
	
	UCSRB&=~(1<<UDRIE);				//wylaczenie przerwania
	while(!(UCSRA&(1<<UDRE)));		//czekanie na zwolnienie interfejsu
	
	UDR=data;						//zapisanie rejestru danych
	
	UCSRB|=UDRIE_state;				//przywrocenie stanu bitu zezw. na przerwanie
	
	SREG=sreg;						//przywrocenie stanu rejestru statusowego
}
////////////////////////////////////////////////////////////////////////////////
//Sprawdzenie, czy w buforze rx sa dane
////////////////////////////////////////////////////////////////////////////////
inline unsigned char UART_data_in_rx_buffer( void )
{
	#ifdef XON_XOFF
	 	if(UART_rx_buffer_empty())
	 	{
	   		UART_transmit_control(UART_XON);
	   								//pobudzenie transmisji
	   								//przydatne w przypadku zablokowania transmisji
	   								//kiedy uklad odbiorczy oczekuje na dane
	   								//a uklad nadawczy jest zablokowany przez XOFF
	 	}
 	#endif
	
   return ( UART_RxHead != UART_RxTail );  
   									//zwroc zero jesli bufor wejsciowy pusty
}
////////////////////////////////////////////////////////////////////////////////
//Sprawdzenie, czy sa dane do przeslania w buforze tx
////////////////////////////////////////////////////////////////////////////////
inline unsigned char UART_data_in_tx_buffer( void )
{
   return ( UART_TxHead != UART_TxTail ); 
   									//zwroc zero jesli bufor wyjsciowy pusty
}
////////////////////////////////////////////////////////////////////////////////
//Czyszczenie buforow
////////////////////////////////////////////////////////////////////////////////
inline void UART_flush_tx_buffer(void)
{
	unsigned char sreg;				//zachowanie stanu rejestru statusowego
	sreg=SREG;
	cli();
		UART_TxHead=0;
		UART_TxTail=0;
	SREG=sreg;
}
inline void UART_flush_rx_buffer(void)
{
	unsigned char sreg;
	sreg=SREG;
	cli();
		UART_RxHead=0;
		UART_RxTail=0;
	SREG=sreg;
}
////////////////////////////////////////////////////////////////////////////////
//Przejscie do nowej linii i dodanie znaku konca stringa (konwencja DOS - \r\n)
////////////////////////////////////////////////////////////////////////////////
void UART_return(void)//przejscie do nowej linii oraz zakonczenie stringa w terminalu
{
UART_transmit_byte('\0');
UART_transmit_byte('\r');
UART_transmit_byte('\n');
}
////////////////////////////////////////////////////////////////////////////////
//Wypisywanie stringa na terminal (wlacznie ze znakiem konca stringa)
////////////////////////////////////////////////////////////////////////////////
unsigned char UART_print(char *tekst)//wypisywanie na terminal
{
unsigned char i=0;

do{
	UART_transmit_byte(*(tekst+i));
	i++;
}while(*(tekst+i));



#ifdef uart_powrot
	UART_return();
#endif

return i;
}

////////////////////////////////////////////////////////////////////////////////
//Odczytywanie stringow lub linii z rsa
////////////////////////////////////////////////////////////////////////////////
unsigned char UART_gets(unsigned char *buf,unsigned char ile)
{
unsigned char i=0;

while(i<ile)
{
	*(buf+i)=UART_receive_byte();

	if((*(buf+i)=='\r')||(*(buf+i)=='\n')||(*(buf+i)=='\0'))
		{
			*(buf+i)='\0';
			return(i+1);	//zwroc ilosc znakow wraz ze znakiem \0
		}
	else
		{
			i++;//przestawienie wskaznika na nowa pozyzje tablicy
		}
}

*(buf+i-1)='\0';
return(i);							//zwracamy liczbe odebranych bajtow
}



///////////////////////////////////////////////////////////////////////////////////////////
void UART_buffer_rx_byte(unsigned char data)
{
	//unsigned char sreg;
	
	
	unsigned char tmphead;
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
	UART_RxHead = tmphead;/* Store new index */
	
	if ( tmphead == UART_RxTail )//obsluga bledu przepelnienia bufora
	{
		UART_flush_rx_buffer(); 
	}

	else
	{
	UART_RxBuf[tmphead] = data; 	//zapisz dane w buforze odbiorczym
	}
}
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
unsigned char UART_buffer_rx_block(unsigned char *tekst)//wypisywanie na terminal
{
unsigned char i=0;
while(/*(*(tekst+i)!='\r')&&*/(*(tekst+i)!='\0'))
{
   UART_buffer_rx_byte(*(tekst+i));
   i++;
}
UART_buffer_rx_byte(*(tekst+i));

return i;
}

