////////////////////////////////////////////////////////////////////////////////
//Copyright (c) 2005
//	Lukasz Matecki
//	lutecki@op.pl
////////////////////////////////////////////////////////////////////////////////
//December 28, 2005 11:02:05 AM 
////////////////////////////////////////////////////////////////////////////////

#ifndef _UART_H_
#define _UART_H_

////////////////////////////////////////////////////////////////////////////////
//Uart
////////////////////////////////////////////////////////////////////////////////

#define UART_RX_BUFFER_SIZE 128 	//2,4,8,16,32,64,128 lub 256 bajtow
#define UART_TX_BUFFER_SIZE 128

#ifdef XON_XOFF
	#define UART_XON 17
	#define UART_XOFF 19
	#define UART_EOT 4
	
	
	volatile unsigned char UART_clear_to_send;
	volatile unsigned char UART_request_to_send;
	volatile unsigned char UART_rx_timeout;
	#define UART_RX_TIMEOUT 10
	
	#define UART_RX_OF_LEVEL 128-32
	#define UART_TX_OF_LEVEL 128-32

	#define UART_RX_UF_LEVEL 16
	#define UART_TX_UF_LEVEL 16
	
	#define UART_rx_buffer_full() (UART_bytes_in_rx_buffer>UART_RX_OF_LEVEL)
	#define UART_tx_buffer_full() (UART_bytes_in_tx_buffer>UART_TX_OF_LEVEL)
	
	#define UART_rx_buffer_empty() (UART_bytes_in_rx_buffer<UART_RX_UF_LEVEL)
	#define UART_tx_buffer_empty() (UART_bytes_in_tx_buffer<UART_TX_UF_LEVEL)
#endif
	
#ifdef _AVR_IOM128_H_
#define UCSRA UCSR0A
#define UCSRB UCSR0B
#define UCSRC UCSR0C
#define UDR UDR0
#define UBRRL UBRR0L
#define SIG_UART_RECV SIG_UART0_RECV
#define SIG_UART_DATA SIG_UART0_DATA
#endif



#define UART_bytes_in_rx_buffer ((UART_RxHead>=UART_RxTail)\
								?(UART_RxHead-UART_RxTail)\
								:(UART_RxHead-UART_RxTail+UART_RX_BUFFER_SIZE))
									
#define UART_bytes_in_tx_buffer ((UART_TxHead>=UART_TxTail)\
								?(UART_TxHead-UART_TxTail)\
								:(UART_TxHead-UART_TxTail+UART_TX_BUFFER_SIZE))


#define UART_CR UART_transmit_byte('\r')
#define UART_LF UART_transmit_byte('\n')
#define UART_NULL UART_transmit_byte('\0')
#define UART_RET UART_transmit_byte('\r');UART_transmit_byte('\n')

#define UART_RX_DISABLE UCSRB&=~(1<<RXEN)
#define UART_RX_ENABLE UCSRB|=(1<<RXEN)
#define UART_TX_DISABLE UCSRB&=~(1<<TXEN)
#define UART_TX_ENABLE UCSRB|=(1<<TXEN)

#define UART_RX_INT_DISABLE UCSRB&=~(1<<RXCIE)
#define UART_RX_INT_ENABLE UCSRB|=(1<<RXCIE)

#define UART_TX_INT_DISABLE UCSRB&=~(1<<UDRIE)
#define UART_TX_INT_ENABLE UCSRB|=(1<<UDRIE)

#define SEND_ESC UART_transmit_byte(27);UART_transmit_byte('[')
#define UART_CLS SEND_ESC;UART_transmit_byte('2');UART_transmit_byte('J')
#define UART_CLEAR_TO_END SEND_ESC;UART_transmit_byte('K')
#define UART_RESET UART_transmit_byte(27);UART_transmit_byte('c')
#define UART_HOME UART_transmit_byte(27);UART_transmit_byte('H')

#define PLAIN_FONT 0
#define BRIGHT_FONT 1
#define DIM_FONT 2
#define UNDERLINED_FONT 4
#define BLINKING_FONT 5
#define REVERSED_FONT 7
#define CONCEALED_FONT 8


#define BLACK_FG_COLOR 30
#define RED_FG_COLOR 31
#define GREEN_FG_COLOR 32
#define YELLOW_FG_COLOR 33
#define BLUE_FG_COLOR 34
#define MAGENTA_FG_COLOR 35
#define CYAN_FG_COLOR 36
#define WHITE_FG_COLOR 37
#define BLACK_BG_COLOR 40
#define RED_BG_COLOR 41
#define GREEN_BG_COLOR 42
#define YELLOW_BG_COLOR 43
#define BLUE_BG_COLOR 44
#define MAGENTA_BG_COLOR 45
#define CYAN_BG_COLOR 46
#define WHITE_BG_COLOR 47

unsigned char uart_echo;


void UART_init(void);
inline unsigned char UART_receive_byte( void );
void UART_buffer_rx_byte(unsigned char data);
unsigned char UART_buffer_rx_block(unsigned char *tekst);
inline void UART_transmit_byte( unsigned char data );
inline void UART_transmit_control( unsigned char data );
inline unsigned char UART_data_in_rx_buffer( void );
inline unsigned char UART_data_in_tx_buffer( void );
unsigned char UART_print(char *tekst);
unsigned char UART_gets(unsigned char *buf,unsigned char ile);
void UART_return(void);

inline void UART_flush_rx_buffer(void);
inline void UART_flush_tx_buffer(void);


void UART_goto(unsigned char row,unsigned char col);

volatile unsigned char UART_TxHead;
volatile unsigned char UART_TxTail;
volatile unsigned char UART_RxHead;
volatile unsigned char UART_RxTail;


#endif
